//=============================================================================
// Battle.cpp
//=============================================================================
#include "Stdinc.h"
#include "Battle.h"
#include "Manipulator.h"
#include "PlayerParty.h"
#include "DataManager.h"
#include "Enemy.h"
#include "Game.h"


//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CBattle::CBattle(void)
	:m_nMode ( BTL_MODE_BATTLE ),m_bEnemySelecting( FALSE ),m_bPlayerSelecting( FALSE ), 
	m_nMarkIndex( CMD_TARGET_NONE ), m_nPlayerWaitStatus( BTL_WAIT_STS_NONE ), m_nEnemyWaitStatus( BTL_WAIT_STS_NONE )
{

}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CBattle::~CBattle(void)
{
	Delete();
}
//-----------------------------------------------------------------------------
// 
//-----------------------------------------------------------------------------
BOOL CBattle::Create( BATTLE_PARAM *pParam )
{
	m_Param = *pParam;
	BOOL bRet = TRUE;

	m_nMode = BTL_MODE_BATTLE;

	// Gp[eB[h
	RemoveAllEnemies();
	CDataManager::GetInstance()->LoadEnemyParty( m_Param.nEnemyPartyID, m_lstEnemy );
	
	// G
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		CEnemy *pEnemy = m_lstEnemy.at( iCnt );
		pEnemy->Create();
		pEnemy->StartAction( CEnemy::ANI_PTN_WALK_AT_START );	// Jns
		LOG_OUTPUT( "[%s] ATB [%d]\n", pEnemy->m_Info.szName, pEnemy->m_Info.nAtbValue );
	}

	// n`G
	bRet &= m_Filed.Create( m_Param.nFiledID );

	// G\
	m_EnemyPartyWnd.Create( 0, 170, 130, 70, m_lstEnemy );
	m_lstCoWindow.push_back( &m_EnemyPartyWnd );

	// v[p[eB\
	m_PlayerPartyWnd.Create( 130, 170, 190, 70, m_Param.pPlayerParty );
	m_lstCoWindow.push_back( &m_PlayerPartyWnd );

	// R}hEBhE
	m_CmdWnd.Create(  25, 170, 90, 70, this );
	m_lstCoWindow.push_back( &m_CmdWnd );

	// TuR}hEBhE
	m_SubWnd.Create(  115, 170, 90, 55, this );
	m_lstCoWindow.push_back( &m_SubWnd );

	// EBhE
	m_InfoWnd.Create(  20, 5, 280, 25 );
	m_lstCoWindow.push_back( &m_InfoWnd );

	// }[N
	m_Mark.Create();
	m_Mark.SetVisible( FALSE );


	// \ʒuݒ
	m_Param.pPlayerParty->SetupBattlePos();

	// ATBl
	for ( int iCnt = 0; iCnt < m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
	{
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( iCnt );
		if ( !pPlayer )
			continue;

		pPlayer->m_CharInfo.nAtbValue = 0;
		pPlayer->SetWaitAction( FALSE );
		LOG_OUTPUT( "[%s] ATB [%d]\n", pPlayer->m_CharInfo.szName, pPlayer->m_CharInfo.nAtbValue );
	}

	// ANVXg
	m_ActList.Initialize();
	m_lstCmdInput.clear();

	// J[\ǗNX
	m_CursorMan.Initialize( m_lstEnemy, m_Param.pPlayerParty, this );

	m_nPlayerWaitStatus = BTL_WAIT_STS_NONE;	// ҂Ă邩
	m_nEnemyWaitStatus = BTL_WAIT_STS_NONE;


	// Đ
	CGame::GetInstance()->StopBGM();

	// TEh[h
	CGame::GetInstance()->LoadSoundFile( BGM_ID_BATTLE );
	CGame::GetInstance()->LoadSoundFile( BGM_ID_BATTLE_WIN );

	// Đ
	CGame::GetInstance()->PlayBGM( BGM_ID_BATTLE, TRUE );

	return bRet;
}

//-----------------------------------------------------------------------------
// I
//-----------------------------------------------------------------------------
void CBattle::Delete( void )
{
	m_Filed.Delete();
	m_EnemyPartyWnd.Delete();
	m_PlayerPartyWnd.Delete();
	m_CmdWnd.Delete();
	m_SubWnd.Delete();
	m_InfoWnd.Delete();

	RemoveAllEnemies();

	m_ActList.Finalize();
	m_CursorMan.Finalize();
	m_Mark.Delete();

	m_lstCoWindow.clear();
}
//-----------------------------------------------------------------------------
// RemoveAllEnemies
//-----------------------------------------------------------------------------
void CBattle::RemoveAllEnemies( void )
{
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		CEnemy *pEnemy = m_lstEnemy.at( iCnt );
		pEnemy->Delete();
		delete pEnemy;
		pEnemy = NULL;
	}
	m_lstEnemy.clear();
}

//-----------------------------------------------------------------------------
// XV
//-----------------------------------------------------------------------------
void CBattle::FrameMove( const CManipulator *pMani )
{
	if ( m_nMode == BTL_MODE_BATTLE )
	{
		FrameMoveModeBattle( pMani );
	}
	else if ( m_nMode == BTL_MODE_WIN )
	{
		FrameMoveModeWon( pMani );
	}
	else if ( m_nMode == BTL_MODE_LOSE )
	{
		FrameMoveModeLose( pMani );
	}
	else if ( m_nMode == BTL_MODE_ESCAPE )
	{
		FrameMoveModeEscape( pMani );
	}
}
//-----------------------------------------------------------------------------
// FrameMoveModeBattle
//-----------------------------------------------------------------------------
void CBattle::FrameMoveModeBattle( const CManipulator *pMani )
{

	m_ActList.FrameMove();

	// J[\
	if ( m_bEnemySelecting || m_bPlayerSelecting )
	{
		m_CursorMan.FrameMove( pMani );
		m_Mark.FrameMove();
		m_PlayerPartyWnd.FrameMove();
	}
	else
	{
		CCoWndManager::FrameMove( pMani );
	}

	// L[`FbN
	if ( pMani->IsKeyPushed( KEY_INPUT_Q ) && pMani->IsKeyPushed( KEY_INPUT_W ) )
	{
		if ( m_Param.pPlayerParty->TryToEscape() )
		{
			LOG_OUTPUT(" ɂ鐬I\n");
			m_nMode = BTL_MODE_ESCAPE;

			m_Mark.SetVisible( FALSE );
			m_CmdWnd.SetVisible( FALSE );
			m_SubWnd.SetVisible( FALSE );
			m_CursorMan.SetVisible( FALSE );

			m_InfoWnd.SetVisible( TRUE );
			m_InfoWnd.SetMessage( "o" );

			return;
		}
	}
	else
	{
		m_Param.pPlayerParty->CancelEscape();
	}



	// 
	for ( int iCnt = 0; iCnt < (int)m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
	{
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer(iCnt);
		if ( !pPlayer )
			continue;

		pPlayer->FrameMoveAtBattle();

		// ATB^`FbN
		if ( iCnt == m_nMarkIndex )
			continue;

		// ĂATB܂
		if ( pPlayer->IsAliving() && pPlayer->IsAtbMax() )
		{
			AddCmdInputList( iCnt );
		}
	}

	// G
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		CEnemy *pEnemy = m_lstEnemy.at( iCnt );
		pEnemy->FrameMove();

		// ATB^`FbN
		if ( pEnemy->IsAtbMax() && ! pEnemy->IsWaitingAction() && !pEnemy->IsActing() )
		{
			// GU҂
			TIMEKEEP_INFO *pInfo = new TIMEKEEP_INFO;
			RtlZeroMemory( pInfo, sizeof(TIMEKEEP_INFO) );
			pInfo->nActType			= TIMEKEEP_ACT_TYPE_ENEMY_CMD_INPUT;
			pInfo->nExecWaitFrame	= TIMEKEEP_WAIT_FRAME_ENEMY_CMD_INPUT;
			pInfo->nParam1			= iCnt;

			m_ActList.Add( CActionList::ACT_ENEMY, pInfo );
			pEnemy->SetWaitAction( TRUE );
		}
	}
	// R}h͑҂v[
	if ( !m_CmdWnd.IsVisible() && !m_ActList.IsSomeoneActing() ) 
	{
		int iIndex = GetActionablePlayer();
		if ( iIndex != -1 )
		{
			CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer(iIndex);
			//if ( !pPlayer->IsWaitingAction() && !pPlayer->IsActing() && 
			// R}h͂
			m_nMarkIndex = iIndex;
			if ( pPlayer )
			{
				// K[hĂ
				pPlayer->SetGuard( FALSE );

				// }[Nʒuݒ
				int nX, nY;
				pPlayer->GetBattlePos( &nX, &nY );
				m_Mark.SetVisible( TRUE );
				m_Mark.SetPos( nX +  2, nY - 12 );

				LOG_OUTPUT( "[%s] R}h\n", pPlayer->m_CharInfo.szName );
				m_CmdWnd.AddCommand( pPlayer->GetCommandList() );
				m_CmdWnd.SetVisible( TRUE );
				ChangeFocus( &m_CmdWnd );
			}
		}
	}

	// ANVXg`FbN
	if ( ! m_ActList.IsSomeoneActing() )	
	{
		TIMEKEEP_INFO *pInfo = m_ActList.PopAction( CActionList::ACT_PLAYER );
		if ( pInfo && pInfo->nActType != TIMEKEEP_ACT_TYPE_NONE )
		{
			DoPlayerAction( pInfo );
			delete pInfo;
		}
	}

	// GANVXg`FbN
	if ( !m_ActList.IsSomeoneActing() )
	{
		TIMEKEEP_INFO *pInfo = m_ActList.PopAction( CActionList::ACT_ENEMY );
		if ( pInfo && pInfo->nActType != TIMEKEEP_ACT_TYPE_NONE )
		{
			DoEnemyAction( pInfo );
			delete pInfo;
		}
	}

	// EFCgXe[^X`FbN
	if ( m_nPlayerWaitStatus == BTL_WAIT_STS_PLAYER_ATTACK_ANI )
	{	// AjI҂
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( m_nActor );
		if ( !pPlayer->IsActing() )
		{
			CEnemy *pEnemy		 = m_lstEnemy.at( m_nTarget );
			// _[WvZ
			int nDamage = CalcAttackDamage( m_nCommandID, pPlayer, pEnemy );

			LOG_OUTPUT("[%s][%d]̃_[W\n", pEnemy->m_Info.szName, nDamage );
			pEnemy->ChangeHitPoint( nDamage );

			pPlayer->StartBattleAction( CPlayerBattle::ANI_PTN_ATTACK_BACK );	// ̃obN
			m_nPlayerWaitStatus = BTL_WAIT_STS_PLAYER_ATTACK_BACK_ANI;
		}
	}

	if ( m_nPlayerWaitStatus == BTL_WAIT_STS_PLAYER_ATTACK_BACK_ANI )
	{	// ̃obNAjI҂
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( m_nActor );
		if ( pPlayer && !pPlayer->IsActing() )
		{
			// ANVsI
			m_ActList.SetSomeoneActing( FALSE );
			LOG_OUTPUT("ANVXg`FbN bN  Actor[%d]\n", m_nActor );
			pPlayer->SetWaitAction( FALSE );
			// ATBlNA
			pPlayer->ClearATBValue();

			m_nPlayerWaitStatus = BTL_WAIT_STS_NONE;
		}
	}

	// GU
	if ( m_nEnemyWaitStatus ==  BTL_WAIT_STS_ENEMY_ATTACK_ANI )
	{
		CEnemy *pEnemy = m_lstEnemy.at( m_nActor );
		if ( !pEnemy->IsActing() )
		{
			CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( m_nTarget );
			if ( pPlayer )
			{
				// _[WvZ
				int nDamage = CalcAttackDamage( m_nCommandID, pPlayer, pEnemy );

				int nEnemyAtack = CSysDebug::GetDebugParam( "Battle", "EnemyAtack", 0 );
				if ( nEnemyAtack == 2 )
					nDamage /= 2;

				LOG_OUTPUT("[%s][%d]̃_[W\n", pPlayer->m_CharInfo.szName, nDamage );
				pPlayer->ChangeHitPoint( nDamage );
				// 񂾂Ƃ̏
				if ( ! pPlayer->IsAliving() && m_nMarkIndex == m_nTarget )
				{
					m_nMarkIndex = CMD_TARGET_NONE;
					m_bEnemySelecting = m_bPlayerSelecting = FALSE;
					m_Mark.SetVisible( FALSE );
					m_CmdWnd.SetVisible( FALSE );
					m_SubWnd.SetVisible( FALSE );

					LOG_OUTPUT( "[%s]퓬s\\n", pPlayer->m_CharInfo.szName );

				}
			}
			m_nEnemyWaitStatus = BTL_WAIT_STS_NONE;
			pEnemy->SetWaitAction( FALSE );
			// ATBlNA
			pEnemy->ClearATBValue();

			m_ActList.SetSomeoneActing( FALSE );
		}
	}
	
	//  GSŃ`FbN
	if ( IsWon() )
	{
		m_nMode = BTL_MODE_WIN;

		// EBhE\
		m_InfoWnd.SetVisible( TRUE );

		// R}hEBhE\
		m_CmdWnd.SetVisible( FALSE );
		m_SubWnd.SetVisible( FALSE );
		m_Mark.SetVisible( FALSE );

		// J[\
		CGame::GetInstance()->GetCursor()->SetVisible( FALSE );

		// TEhĐ
		CGame::GetInstance()->PlayBGM( BGM_ID_BATTLE_WIN, TRUE );

		// уWv [VX^[g
		for ( int iCnt = 0; iCnt < m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
		{
			CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( iCnt );
			if ( !pPlayer )
				continue;
			pPlayer->StartBattleAction( CPlayerBattle::ANI_PTN_WIN_JUMP );
		}

		// ݒ
		SetWinInfo();
	}
	// SŃ`FbN
	if ( IsLose() )
	{
		// J[\
		CGame::GetInstance()->GetCursor()->SetVisible( FALSE );

		m_nMode = BTL_MODE_LOSE;
		m_InfoWnd.SetVisible( TRUE );
		m_InfoWnd.SetMessage( "Sł" );
	}
}
//-----------------------------------------------------------------------------
// SetWinInfo
//-----------------------------------------------------------------------------
void CBattle::SetWinInfo( void )
{
	int nGill = 0;
	int nExp = 0;
	int nABP = 0;

	// Gv
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		CEnemy *pEnemy = m_lstEnemy.at( iCnt );
		nGill += pEnemy->m_Info.nGill;
		nExp  += pEnemy->m_Info.nExp;
		nABP  += pEnemy->m_Info.nABP;
	}

	// Mǉ
	m_Param.pPlayerParty->AddGill( nGill );

	// olABP
	int nAliveCount = m_Param.pPlayerParty->GetAlivingPlayerCount();
	int nExpPerPlayer = (int)nExp / nAliveCount;
	int nABPPerPlayer = (int)nABP / nAliveCount;

	if ( nExp != 0 && nExpPerPlayer == 0 )
		nExpPerPlayer = 1; 
	if ( nABP != 0 && nABPPerPlayer == 0 )
		nABPPerPlayer = 1;

	// ǉbZ[W
	std::list<char *>	aryMsgList;
	char *pMessage;

	for ( int iCnt = 0; iCnt < m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
	{
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( iCnt );
		if ( !pPlayer )
			continue;

		if ( pPlayer->IsAliving() )
		{	// Ă郁oŊZ
			if ( pPlayer->AddExp( nExpPerPlayer ) )
			{	// LvAbv
				pMessage = new char [DISPLAY_MESSAGE_LEN_MAX];
				sprintf_s( pMessage, DISPLAY_MESSAGE_LEN_MAX, "%skuAbvI", pPlayer->m_CharInfo.szName );
				aryMsgList.push_back( pMessage );

			}
			if ( pPlayer->AddABP( nABPPerPlayer ) )
			{	// JobLvAbv
				pMessage = new char [DISPLAY_MESSAGE_LEN_MAX];
				sprintf_s( pMessage, DISPLAY_MESSAGE_LEN_MAX, "%sWukuAbvI", pPlayer->m_CharInfo.szName );
				aryMsgList.push_back( pMessage );
			}
		}
	}

	m_InfoWnd.SetWinInfo( nGill, nExp, nABP, aryMsgList );
}
//-----------------------------------------------------------------------------
// FrameMoveModeWon
//-----------------------------------------------------------------------------
void CBattle::FrameMoveModeWon( const CManipulator *pMani )
{
	m_InfoWnd.FrameMove( pMani );
	
	// 
	for ( int iCnt = 0; iCnt < (int)m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
	{
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer(iCnt);
		if ( !pPlayer )
			continue;

		pPlayer->FrameMoveAtBattle();
	}
	if ( !m_InfoWnd.IsVisible() )
	{	// 퓬I
		//m_nMode = BTL_MODE_BATTLE;	// [h
		CGame::GetInstance()->FinishedBattle();
	}
}
//-----------------------------------------------------------------------------
// FrameMoveModeLose
//-----------------------------------------------------------------------------
void CBattle::FrameMoveModeLose( const CManipulator *pMani )
{
	m_InfoWnd.FrameMove( pMani );

	// 
	for ( int iCnt = 0; iCnt < (int)m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
	{
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer(iCnt);
		if ( !pPlayer )
			continue;

		pPlayer->FrameMoveAtBattle();
	}

	if ( pMani->CheckKey( KEY_INPUT_X ) )
	{
		// for debug
		for ( int iCnt = 0; iCnt < (int)m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
		{
			CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer(iCnt);
			if ( !pPlayer )
				continue;

			pPlayer->RecoveryHPMP();
		}
		CGame::GetInstance()->FinishedBattle();
	}

}
//-----------------------------------------------------------------------------
// FrameMoveModeEscape
//-----------------------------------------------------------------------------
void CBattle::FrameMoveModeEscape( const CManipulator *pMani )
{
	m_InfoWnd.FrameMove( pMani );

	// 
	for ( int iCnt = 0; iCnt < (int)m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
	{
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer(iCnt);
		if ( !pPlayer )
			continue;
		if ( pPlayer->IsActing() )
		{	// AjI܂
			pPlayer->FrameMoveAtBattle();
		}
	}
	// G
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		CEnemy *pEnemy = m_lstEnemy.at( iCnt );
		if ( !pEnemy )
			continue;
		pEnemy->FrameMove();
	}

	if ( pMani->CheckKey( KEY_INPUT_X ) )
	{
		CGame::GetInstance()->FinishedBattle();
	}
}

//-----------------------------------------------------------------------------
// IsWon
//-----------------------------------------------------------------------------
BOOL CBattle::IsWon( void )
{
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		if ( m_lstEnemy.at(iCnt)->IsVisible() )
		{
			return FALSE;
		}
	}

	return TRUE;
}
//-----------------------------------------------------------------------------
// IsLose
//-----------------------------------------------------------------------------
BOOL CBattle::IsLose( void )
{
	for ( int iCnt = 0; iCnt < m_Param.pPlayerParty->GetPlayerCount(); iCnt ++ )
	{
		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( iCnt );
		if ( !pPlayer )
			continue;

		if ( pPlayer->IsAliving() )
		{
			return FALSE;
		}
	}

	return TRUE;
}
//-----------------------------------------------------------------------------
// `
//-----------------------------------------------------------------------------
void CBattle::Render( void )
{
	// tB[h
	m_Filed.Render();

	// 
	m_Param.pPlayerParty->Render();

	// G
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		m_lstEnemy.at( iCnt )->Render();
	}

	// EBhE
	CCoWndManager::Render();

	// I
	if ( m_bEnemySelecting || m_bPlayerSelecting )
	{
		m_CursorMan.Render();	// J[\
	}

	m_Mark.Render();		// }[N

}
//-----------------------------------------------------------------------------
// SetVisible
//-----------------------------------------------------------------------------
void CBattle::SetVisible( BOOL bFlag )
{
	CCoWndManager::SetVisible( bFlag );
	if ( bFlag )
	{
		m_CmdWnd.SetVisible( FALSE );
		m_SubWnd.SetVisible( FALSE );
		m_InfoWnd.SetVisible( FALSE );
	}
}
//-----------------------------------------------------------------------------
// qEBhẼj[ʒm
//-----------------------------------------------------------------------------
void CBattle::NotifyDecideKey( CFrameWindow *pWnd, int nCmdID )
{
	if ( pWnd == &m_CmdWnd )
	{	// CR}hEBhE

		// ToDo:
		switch( nCmdID )
		{
		case COMMAND_ID_ATTACK:	// 
			{
				m_bEnemySelecting = TRUE;
				LOG_OUTPUT("GIJn\n");
				m_CursorMan.AdjustEnemyCursorIndex();
				LOG_OUTPUT("GI\n" );
			}
			break;
		case COMMAND_ID_ITEM:
			break;
		}
	}
	if ( pWnd == &m_SubWnd )
	{	// TuR}hEBhE

		CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( m_nMarkIndex );
		if ( !pPlayer || !pPlayer->IsAliving() )
			return;

		switch( nCmdID )
		{
		case COMMAND_ID_GUARD:		// ڂ
			pPlayer->SetGuard( TRUE );
			m_CmdWnd.SetVisible( FALSE );
			m_SubWnd.SetVisible( FALSE );
			m_Mark.SetVisible( FALSE );
			break;
		case COMMAND_ID_PASS:		// pX
			m_CmdWnd.SetVisible( FALSE );
			m_SubWnd.SetVisible( FALSE );
			m_Mark.SetVisible( FALSE );

			break;
		case COMMAND_ID_CHANGE:		// `FW
			break;
		default:
			break;
		}
	}
	else if ( pWnd == NULL )
	{	// J[\ǗNX

		if ( m_bEnemySelecting && m_nMarkIndex != CMD_TARGET_NONE )		// m_nMarkIndex == CMD_TARGET_NONE  R}h̓LS
		{	// GI  Us

			LOG_OUTPUT( "GI  Us [%d]\n", m_nMarkIndex );

			TIMEKEEP_INFO *pInfo = new TIMEKEEP_INFO;
			RtlZeroMemory( pInfo, sizeof(TIMEKEEP_INFO) );
			pInfo->nActType			= TIMEKEEP_ACT_TYPE_CMD;
			pInfo->nExecWaitFrame	= TIMEKEEP_WAIT_FRAME_CMD_EXEC;
			pInfo->nCommandID		= COMMAND_ID_ATTACK;
			pInfo->nParam1			= m_nMarkIndex;	// 
			pInfo->nParam2			= nCmdID;		// G
			

			m_ActList.Add( CActionList::ACT_PLAYER, pInfo );
			m_bEnemySelecting = FALSE;
			m_CmdWnd.SetVisible( FALSE );
			m_Mark.SetVisible( FALSE );

			CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( pInfo->nParam1 );
			if ( pPlayer )
				pPlayer->SetWaitAction( TRUE );

		}

	}
}
//-----------------------------------------------------------------------------
// qEBhẼLZ{^ʒm
//-----------------------------------------------------------------------------
void CBattle::NotifyCancelKey( CFrameWindow *pWnd )
{
	if ( pWnd == &m_SubWnd )
	{	// TuR}hEBhE
		m_SubWnd.SetVisible( FALSE );
		ChangeFocus( &m_CmdWnd );
	}
	else if ( pWnd == NULL )
	{	// J[\ǗNX

		if ( m_bEnemySelecting )
		{	// GI𒆒f
			m_bEnemySelecting = FALSE;
			ChangeFocus( &m_CmdWnd );
		}
		else if ( m_bPlayerSelecting )
		{	// I𒆒f
			m_bPlayerSelecting = FALSE;
			ChangeFocus( &m_CmdWnd );
		}
	}
}
//-----------------------------------------------------------------------------
// CR}hEBhẺEL[
//-----------------------------------------------------------------------------
void CBattle::NotifyRightKey( CFrameWindow *pWnd )
{
	if ( pWnd == &m_CmdWnd )
	{
		m_SubWnd.SetVisible( TRUE );
		ChangeFocus( &m_SubWnd );
	}
}
//-----------------------------------------------------------------------------
// TuR}hEBhẺEL[
//-----------------------------------------------------------------------------
void CBattle::NotifyLeftKey( CFrameWindow *pWnd )
{
	if ( pWnd == &m_SubWnd )
	{
		m_SubWnd.SetVisible( FALSE );
		ChangeFocus( &m_CmdWnd );
	}
}
//-----------------------------------------------------------------------------
// DoAction
//-----------------------------------------------------------------------------
void CBattle::DoPlayerAction( TIMEKEEP_INFO *pInfo )
{
	LOG_OUTPUT("DoPlayerAction %s\n", CSysDebug::GetActString( pInfo->nActType ) );

	switch ( pInfo->nActType )
	{
	case TIMEKEEP_ACT_TYPE_CMD:		// R}h s
		{	// nParam1  nParam2ւ

			CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( pInfo->nParam1 );
			if ( pPlayer )
			{
				CEnemy *pEnemy = m_lstEnemy.at( pInfo->nParam2 );
				if ( !pEnemy || !pEnemy->IsAliving() )
				{	// GɂȂ
					LOG_OUTPUT( "[%d] Ȃ ΏۍĐݒ\n", pInfo->nParam2 );
					pInfo->nParam2 = AutoSelectEnemyTarget();
					if ( pInfo->nParam2 == CMD_TARGET_NONE  )
					{	// GɂȂEEE蓾Ȃ
						return;
					}
				}

				LOG_OUTPUT( "[%s]  [%d] \n", pPlayer->m_CharInfo.szName, pInfo->nParam2 );
				// Aj[Vݒ
				pPlayer->StartBattleAction( CPlayerBattle::ANI_PTN_ATTACK );	// 
				m_nPlayerWaitStatus = BTL_WAIT_STS_PLAYER_ATTACK_ANI;	// Aj[VI҂
				m_nCommandID = pInfo->nCommandID;	// U
				m_nActor     = pInfo->nParam1;
				m_nTarget    = pInfo->nParam2;

				// ANVsI܂ő̃v[̃ANVXg̓`FbNȂ
				m_ActList.SetSomeoneActing( TRUE );
				LOG_OUTPUT("ANVXg`FbN bN Actor[%d]\n", m_nActor );
			}
		}
		break;
	}


}
//-----------------------------------------------------------------------------
// DoEnemyAction
//-----------------------------------------------------------------------------
void CBattle::DoEnemyAction( TIMEKEEP_INFO *pInfo )
{
	LOG_OUTPUT("DoAction %s\n", CSysDebug::GetActString( pInfo->nActType ) );

	switch ( pInfo->nActType )
	{

	case TIMEKEEP_ACT_TYPE_ENEMY_CMD_INPUT:	// GR}hI
		{
			CEnemy *pEnemy = m_lstEnemy.at( pInfo->nParam1 );
			if ( pEnemy->IsAliving() )
			{
				TIMEKEEP_INFO *pNewAction = new TIMEKEEP_INFO;
				pEnemy->DecideAction( pNewAction, m_Param.pPlayerParty );
				pNewAction->nParam1 = pInfo->nParam1;

				LOG_OUTPUT( "[%s]  [%d] \n", pEnemy->m_Info.szName, pNewAction->nParam2 );
				m_ActList.Add( CActionList::ACT_ENEMY, pNewAction );

			}
		}
		break;
	case TIMEKEEP_ACT_TYPE_ENEMY_CMD:	// GR}hs
		{

			int nEnemyAtack = CSysDebug::GetDebugParam( "Battle", "EnemyAtack", 0 );
			// ANVs
			m_ActList.SetSomeoneActing( TRUE );
			LOG_OUTPUT("ANVXg`FbN bN(GR}hs) nActor [%d]\n", m_nActor );
			if ( nEnemyAtack == 1 )
			{
				m_ActList.SetSomeoneActing( FALSE );
				return;
			}

			CEnemy *pEnemy = m_lstEnemy.at( pInfo->nParam1 );
			pEnemy->StartAction( CEnemy::ANI_PTN_ATTACK );	// 
			m_nEnemyWaitStatus = BTL_WAIT_STS_ENEMY_ATTACK_ANI;	// GAj[VI҂
			m_nCommandID = pInfo->nCommandID;	// U
			m_nActor     = pInfo->nParam1;
			m_nTarget    = pInfo->nParam2;

			CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( m_nTarget );
			if ( !pPlayer || !pPlayer->IsAliving() )
			{	// ^[QbgłIђ
				m_nTarget = pEnemy->SelectAttackTarget( m_Param.pPlayerParty );
			}
		}
		break;
	}
}

//-----------------------------------------------------------------------------
// ΏۓGI
//-----------------------------------------------------------------------------
int CBattle::AutoSelectEnemyTarget( void )
{
	std::vector<int> aryEnemyIndex;
	// ĂG 
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		CEnemy *pEnemy = m_lstEnemy.at( iCnt );
		if ( !pEnemy )
			continue;

		if ( pEnemy->IsAliving() )
		{
			aryEnemyIndex.push_back( iCnt );
		}
	}

	if ( aryEnemyIndex.size() <= 0 )
		return CMD_TARGET_NONE;	// ^[Qbg

	int nRand = GetRand( aryEnemyIndex.size() * 1000 ) % aryEnemyIndex.size();

	return aryEnemyIndex.at( nRand );
}
//-----------------------------------------------------------------------------
// CalcAttackDamage
//-----------------------------------------------------------------------------
int CBattle::CalcAttackDamage( int nCommandID, CPlayerChar *pPlayer, CEnemy *pEnemy )
{
	int nDamage = 0;

	if ( nCommandID == COMMAND_ID_ATTACK )
	{	// v[̍U
		nDamage = pPlayer->m_Param.nAttack - ( pEnemy->m_Param.nDefence / 2 );
	}
	else if ( nCommandID == COMMAND_ID_ENEMY_ATTACK )
	{	// G̍U
		nDamage = pEnemy->m_Param.nAttack - ( pPlayer->m_Param.nDefence / 2 );
	}

	if ( nDamage < 0 )
		nDamage = 0;

	return nDamage;
}
//-----------------------------------------------------------------------------
// AddCmdInputList
//-----------------------------------------------------------------------------
void CBattle::AddCmdInputList( int nPlayerIndex )
{
	// łɓo^ĂȂΓo^
	std::list<int>::iterator it = m_lstCmdInput.begin();
	while( it != m_lstCmdInput.end() )
	{
		if ( nPlayerIndex == (*it) )
		{	// łɓo^Ă Ȃ
			return;
		}
		++it;
	}
	m_lstCmdInput.push_back( nPlayerIndex );
}
//-----------------------------------------------------------------------------
// GetActionablePlayer
//-----------------------------------------------------------------------------
int CBattle::GetActionablePlayer( void )
{
	if ( m_lstCmdInput.empty() )
		return -1;	// o^Ȃ

	int nIndex = m_lstCmdInput.front();
	m_lstCmdInput.pop_front();

	CPlayerChar *pPlayer = m_Param.pPlayerParty->GetPlayer( nIndex );
	if ( !pPlayer || !pPlayer->IsAliving() )
		return -1;	// ł

	return nIndex;
}