//=============================================================================
// BattleCursorManager.cpp
//=============================================================================
#include <algorithm>
#include "Stdinc.h"
#include "BattleCursorManager.h"
#include "Game.h"
#include "Cursor.h"
#include "Manipulator.h"
#include "Enemy.h"
#include "PlayerParty.h"

//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CBattleCursorManager::CBattleCursorManager(void)
	:m_nSelMode( SELECT_MODE_ENEMY )
{
	m_pCursor = CGame::GetInstance()->GetCursor();
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CBattleCursorManager::~CBattleCursorManager(void)
{
}
//-----------------------------------------------------------------------------
// Initialize
//-----------------------------------------------------------------------------
BOOL CBattleCursorManager::Initialize( std::vector<CEnemy *> &lstEnemy, CPlayerParty *pPlayerParty, CBattle *pParent )
{
	// GXgRs[
	m_lstEnemy.assign( lstEnemy.begin(),lstEnemy.end() );
	std::copy(lstEnemy.begin(),lstEnemy.end(),m_lstEnemy.begin());

	// v[p[eBRs[
	m_pPlayerParty = pPlayerParty;

	m_pParent = pParent;

	// J[\ʒuݒ
	m_nDispPosX = CANVAS_WIDTH;
	m_nDispPosY = CANVAS_HEIGHT;

	SetEnemyCursorOnKey( KEY_INPUT_LEFT );

	return TRUE;
}
//-----------------------------------------------------------------------------
// Finalize
//-----------------------------------------------------------------------------
void CBattleCursorManager::Finalize( void )
{
}
//-----------------------------------------------------------------------------
// J[\\
//-----------------------------------------------------------------------------
void CBattleCursorManager::SetVisible( BOOL bFlag )
{
	m_pCursor->SetVisible( bFlag );
};
//-----------------------------------------------------------------------------
// XV
//-----------------------------------------------------------------------------
void CBattleCursorManager::FrameMove( const CManipulator *pMani )
{
	if ( m_nSelMode == SELECT_MODE_ENEMY )
	{	// GI

		// L[
		if ( pMani->CheckKey( KEY_INPUT_UP ) )
		{
			SetEnemyCursorOnKey( KEY_INPUT_UP );
		}
		if ( pMani->CheckKey( KEY_INPUT_DOWN ) )
		{
			SetEnemyCursorOnKey( KEY_INPUT_DOWN );
		}
		if ( pMani->CheckKey( KEY_INPUT_LEFT ) )
		{
			SetEnemyCursorOnKey( KEY_INPUT_LEFT );
		}
		if ( pMani->CheckKey( KEY_INPUT_RIGHT ) )
		{
			SetEnemyCursorOnKey( KEY_INPUT_RIGHT );
		}

		// {^
		if ( pMani->CheckKey( KEY_INPUT_X ) )
		{
			m_pParent->NotifyDecideKey( NULL, m_nEnemyCursorIndex );
		}
		// LZ{^
		else if ( pMani->CheckKey( KEY_INPUT_Z ) )
		{
			m_pParent->NotifyCancelKey( NULL );
		}
	}
}
//-----------------------------------------------------------------------------
// `
//-----------------------------------------------------------------------------
void CBattleCursorManager::Render( void )
{
	m_pCursor->Render();
}
//-----------------------------------------------------------------------------
// L[͂ɂJ[\ʒuݒ
//-----------------------------------------------------------------------------
BOOL CBattleCursorManager::SetEnemyCursorOnKey( int nKeyCode )
{
	m_nSelMode = SELECT_MODE_ENEMY;	// GI

	int nIndex = -1;
	int nDiffY = CANVAS_HEIGHT;
	int nDiffX = CANVAS_WIDTH;
	for ( int iCnt = 0; iCnt < (int)m_lstEnemy.size(); iCnt ++ )
	{
		CEnemy *pEnemy = m_lstEnemy.at( iCnt );

		if ( !pEnemy || pEnemy->m_Info.nHP <= 0 )
			continue;	// HP 0̓G͏O

		if ( iCnt == m_nEnemyCursorIndex )
			continue;	// IĂ̂O

		int nEnemyPosX, nEnemyPosY;
		pEnemy->GetCursorDispPos( &nEnemyPosX, &nEnemyPosY );

		if ( nKeyCode == KEY_INPUT_LEFT || nKeyCode == KEY_INPUT_UP )
		{	// 
			if ( m_nDispPosX - nEnemyPosX < nDiffX && 0 <= m_nDispPosX - nEnemyPosX )
			{
				nDiffX = m_nDispPosX - nEnemyPosX;
				nIndex = iCnt;
			}
		}
		if ( nKeyCode == KEY_INPUT_RIGHT || nKeyCode == KEY_INPUT_DOWN )
		{	// E
			if ( nEnemyPosX - m_nDispPosX < nDiffX && 0 <= nEnemyPosX - m_nDispPosX )
			{
				nDiffX = nEnemyPosX - m_nDispPosX;
				nIndex = iCnt;
			}
		}

	}
	if ( nIndex == -1 )
	{	// 
		LOG_OUTPUT("J[\ω\n");
		return FALSE;
	}

	m_nEnemyCursorIndex = nIndex;
	m_lstEnemy.at( m_nEnemyCursorIndex )->GetCursorDispPos( &m_nDispPosX, &m_nDispPosY );
	m_pCursor->SetPos( m_nDispPosX, m_nDispPosY );

	return TRUE;
}
//-----------------------------------------------------------------------------
// J[\CfbNX
//-----------------------------------------------------------------------------
void CBattleCursorManager::AdjustEnemyCursorIndex( void )
{
	m_nSelMode = SELECT_MODE_ENEMY;	// GI

	// J[\ʒu̓GĂ邩
	CEnemy *pEnemy = m_lstEnemy.at( m_nEnemyCursorIndex );
	if ( pEnemy && 0 < pEnemy->m_Info.nHP )
	{
		m_lstEnemy.at( m_nEnemyCursorIndex )->GetCursorDispPos( &m_nDispPosX, &m_nDispPosY );
		m_pCursor->SetPos( m_nDispPosX, m_nDispPosY );
		return;	// sv
	}

	// 
	if ( !SetEnemyCursorOnKey( KEY_INPUT_RIGHT ) )
	{
		SetEnemyCursorOnKey( KEY_INPUT_LEFT );
	}

	m_pCursor->SetVisible( TRUE );
}