//=============================================================================
// Enemy.cpp
//=============================================================================
#include "Stdinc.h"
#include "Enemy.h"
#include <assert.h>
#include "Battle.h"
#include "PlayerParty.h"
//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CEnemy::CEnemy(void)
	:m_nFrameCount( 0 ), m_bWaitAction( FALSE ), m_nAnimationPtrn( ANI_PTN_NONE ), m_bDrawBlink( FALSE )
{
	RtlZeroMemory( &m_Info,		sizeof(m_Info) );
	RtlZeroMemory( &m_Param,    sizeof(m_Param) );
	RtlZeroMemory( &m_Status,   sizeof(m_Status) );
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CEnemy::~CEnemy(void)
{
}
//-----------------------------------------------------------------------------
// Create
//-----------------------------------------------------------------------------
BOOL CEnemy::Create( void )
{
	assert( m_Info.nID );

	char szFileName[MAX_PATH];
	sprintf_s( szFileName, sizeof(szFileName), "%s\\%s%03d.png", ENEMY_GRAPHIC_DIR, ENEMY_FNAME_PREFIX, m_Info.nID );

	m_nHandle = ::LoadGraph( szFileName );
	if ( m_nHandle == DX_FAILED )
	{
		CSysUtil::Msg("摜ǂݍݎs [%s]", szFileName );
		return FALSE;
	}
	GetGraphSize( m_nHandle, &m_nGraphWidth, &m_nGraphHeight );

	m_bWaitAction = FALSE;
	m_nFrameCount = 0;

	return TRUE;
}
//-----------------------------------------------------------------------------
// FrameMove
//-----------------------------------------------------------------------------
void CEnemy::FrameMove( void )
{
	m_nFrameCount ++;

	int nEnemyATBSpeed = CSysDebug::GetDebugParam( "Battle", "EnemyATBSpeed", 0 );

	int nRate;
	if ( nEnemyATBSpeed == 1 )
	{	// 
		nRate = 10;
	}
	else if ( nEnemyATBSpeed == 2 )
	{	// ͂₢
		nRate = 2;
	}
	else if ( nEnemyATBSpeed == 3 || nEnemyATBSpeed == 4 )
	{	//  / 
		nRate = 1;
	}
	else
	{	// ʏ
		nRate = 6;
	}

	if ( m_nFrameCount % nRate == 0 )
	{
		// Xe[^Xɉ ATBlXV
		if ( m_Info.nAtbValue < ATB_VALUE_MAX )
		{
			++ m_Info.nAtbValue;
			if (  nEnemyATBSpeed == 4 )
			{
				++ m_Info.nAtbValue;
				if ( ATB_VALUE_MAX < m_Info.nAtbValue )
				{
					m_Info.nAtbValue = ATB_VALUE_MAX;
				}
			}
		}
		else
		{
			m_Info.nAtbValue = ATB_VALUE_MAX;
		}

	}

	m_NumAnime.FrameMove();	// lAj[V


	// Aj[V
	if ( m_nAniFrameCount == 0 )
		return;

	m_nAniFrameCount --;	// Aj[VJE^ (fNg)

	// Aj[V
	switch ( m_nAnimationPtrn )
	{
	case ANI_PTN_NONE:			// Aj[V
		m_nDrawOffsetX = 0;
		m_nDrawOffsetY = 0;
		break;

	case ANI_PTN_WALK_AT_START:	// Jns
		m_nDrawOffsetX += 2;
		break;
	case ANI_PTN_ATTACK:		// 
		m_bDrawBlink = ( m_nAniFrameCount % 6 < 2 );
		break;
	}

	// ŏIt[Œ~
	if ( m_nAniFrameCount == 0 )
	{
		StopAnimation();
	}
}
//-----------------------------------------------------------------------------
// StopAnimation
//-----------------------------------------------------------------------------
void CEnemy::StopAnimation( void )
{
	m_nAnimationPtrn = ANI_PTN_NONE;
	m_nDrawOffsetX = 0;
	m_nDrawOffsetY = 0;
	m_bDrawBlink = FALSE;
}
//-----------------------------------------------------------------------------
// StartAction
//-----------------------------------------------------------------------------
void CEnemy::StartAction( int nAniPtnNo )
{
	m_nAnimationPtrn = nAniPtnNo;
	m_nDrawOffsetX = 0;
	m_nDrawOffsetY = 0;

	switch ( m_nAnimationPtrn )
	{
	case ANI_PTN_WALK_AT_START:	// Jns
		m_nAniFrameCount =  24;
		m_nDrawOffsetX   = -48;
		break;
	case ANI_PTN_ATTACK:		// 
		m_nAniFrameCount = 20;
		break;
	}

}
//-----------------------------------------------------------------------------
// Render
//-----------------------------------------------------------------------------
void CEnemy::Render( void )
{
	if ( !IsAliving() && !m_NumAnime.IsVisible() )
	{
		SetVisible( FALSE );
		return;
	}

	// _ŕ`
	if ( m_bDrawBlink )
	{
		SetDrawBright( 128 , 128 , 128 );
		DrawGraph( m_nPosX + m_nDrawOffsetX, m_nPosY + m_nDrawOffsetY, m_nHandle , TRUE );
		SetDrawBright( 255 , 255 , 255 );
	}
	else
	{	// ʏ`
		DrawGraph( m_nPosX + m_nDrawOffsetX, m_nPosY + m_nDrawOffsetY, m_nHandle, TRUE );
	}

	m_NumAnime.Render();	// lAj[V
}
//-----------------------------------------------------------------------------
// Delete
//-----------------------------------------------------------------------------
void CEnemy::Delete( void )
{
	DeleteGraph( m_nHandle );
}
//-----------------------------------------------------------------------------
// DecideAction
//-----------------------------------------------------------------------------
BOOL CEnemy::DecideAction( TIMEKEEP_INFO *pNewAction, CPlayerParty *pPlayerParty )
{
	RtlZeroMemory( pNewAction, sizeof(TIMEKEEP_INFO) );

	pNewAction->nActType		= TIMEKEEP_ACT_TYPE_ENEMY_CMD;				// G̃R}htF[Y
	pNewAction->nExecWaitFrame	= TIMEKEEP_WAIT_FRAME_ENEMY_EXEC;

	pNewAction->nCommandID		= COMMAND_ID_ENEMY_ATTACK;	// ToDo: Œ

	// UΏېݒ
	//pNewAction->nParam1 Index (OŐݒ肷)
	pNewAction->nParam2			= SelectAttackTarget( pPlayerParty );

	return TRUE;
}
//-----------------------------------------------------------------------------
// SelectAttackTarget
//-----------------------------------------------------------------------------
int CEnemy::SelectAttackTarget( CPlayerParty *pPlayerParty )
{
	std::vector<int> aryPlayerIndex;
	// Ăv[
	for ( int iCnt = 0; iCnt < pPlayerParty->GetPlayerCount(); iCnt ++ )
	{
		CPlayerChar *pPlayer = pPlayerParty->GetPlayer(iCnt);
		if ( !pPlayer )
			continue;

		if ( pPlayer->IsAliving() )
		{
			aryPlayerIndex.push_back( iCnt );
		}
	}

	if ( aryPlayerIndex.size() <= 0 )
		return CMD_TARGET_NONE;	// ^[Qbg

	int nRand = GetRand( aryPlayerIndex.size() * 1000 ) % aryPlayerIndex.size();

	return aryPlayerIndex.at( nRand );
}
//-----------------------------------------------------------------------------
// GetCursorDispPos
// J[\\ʒu擾
//-----------------------------------------------------------------------------
void CEnemy::GetCursorDispPos( int *pX, int *pY )
{
	*pX = m_nPosX - 8;
	*pY = m_nPosY + (m_nGraphHeight / 2) - 8;
}
//-----------------------------------------------------------------------------
// GetNumAnimeDispPos
// Aj[V\ʒu擾
//-----------------------------------------------------------------------------
void CEnemy::GetNumAnimeDispPos( int *pX, int *pY, int iDispWidth )
{
	*pX = m_nPosX + ((m_nGraphWidth - iDispWidth) / 2);
	*pY = m_nPosY + m_nGraphHeight - 12;
}
//-----------------------------------------------------------------------------
// ChangeHitPoint
// HP
//-----------------------------------------------------------------------------
void CEnemy::ChangeHitPoint( int iDiff )
{
	if ( m_Info.nHP == 0 && iDiff < 0 )
	{
		// 퓬s\ɉ񕜂͂łȂ
		return;
	}

	// HPAj[Vݒ
	DWORD dwColor;
	if ( 0 <= iDiff )
	{	// 0Ɛ̒l
		dwColor = GetColor(255,255,255);
	}
	else
	{	// ̒l(HP)
		dwColor = GetColor(96,255,96);
	}

	int iX,iY;
	int iWidth = CNumberAnimation::GetDispWidth( abs(iDiff) );
	GetNumAnimeDispPos( &iX, &iY, iWidth );
	m_NumAnime.StartAction( iX, iY, abs(iDiff), dwColor );

	// HP炷
	m_Info.nHP -= iDiff;
	if ( m_Info.nHP < 0 )
		m_Info.nHP = 0;

	if ( m_Param.Base.nMaxHP < m_Info.nHP )
	{
		m_Info.nHP = m_Param.Base.nMaxHP;
	}
	LOG_OUTPUT( "[%s] cHP [%d]\n", m_Info.szName, m_Info.nHP );
}