//=============================================================================
// Game.cpp
//=============================================================================
#include "Stdinc.h"
#include <time.h>
#include "Game.h"
#include "DataManager.h"
#include "MapObject.h"
#include "PlayerParty.h"
#include "Item.h"
#include "UseItem.h"

//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CGame::CGame(void)
: m_nWaitStatus( WAIT_STS_NONE ), m_timPlayTime( 0 ), m_timLastPlayTime( 0 ), m_bEventBattle( FALSE ), m_nLastAnswer( ANSWER_NONE )
{
	RtlZeroMemory( m_Flag.m_chGame, sizeof(m_Flag.m_chGame) );
	RtlZeroMemory( m_Flag.m_chTreasure, sizeof(m_Flag.m_chTreasure) );

	m_lstMode.clear();

}
//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CGame::~CGame(void)
{
}
//-----------------------------------------------------------------------------
// GetInstance
//-----------------------------------------------------------------------------
CGame *CGame::GetInstance( void )
{
	static CGame m_Instance;
	return &m_Instance;
}
//-----------------------------------------------------------------------------
// Initialize
//-----------------------------------------------------------------------------
BOOL CGame::Initialize( void )
{
	BOOL bRet = TRUE;

	// vCJnԏ
	time( &m_timStartTime );

	// LN^Gǂݍ
	bRet &= CDataManager::GetInstance()->LoadCharacterGraph();

	// }bv
	bRet &= m_Map.Initialize();

	// p[eB쐬
	m_PlayerParty.Initialize();

	// J[\쐬
	m_Cursor.Create();
	m_Cursor.SetVisible( FALSE );

	// ACegp
	CUseItem::GetInstance()->Initialize( &m_Battle, this, &m_PlayerParty );

	//  EBhE쐬
	// bZ[WEBhE
	m_wndMsg.Create(  20/*X*/, 5/*Y*/, 280/**/, 72/**/ );
	m_wndMsg.ShowWindow( FALSE );

	// Yes/No EBhE
	m_wndYesNo.Create(  20/*X*/, 5/*Y*/, 280/**/, 72/**/ );
	m_wndYesNo.ShowWindow( FALSE );

	// EBhE
	m_wndChoice.Create(  20/*X*/, 5/*Y*/, 280/**/, 72/**/ );
	m_wndChoice.ShowWindow( FALSE );

	// LvEBhE
	m_wndCamp.Create( &m_PlayerParty );	// S
	m_wndCamp.SetVisible( FALSE );

	// XEBhE
	m_wndStore.Create( &m_PlayerParty );	// S
	m_wndStore.SetVisible( FALSE );

	// TEh
	m_Sound.Initialize();

	// GJEg
	m_Encount.Initialize( &m_Map );

	// ToDo:
	DEBUG_init();

	return bRet;
}
//-----------------------------------------------------------------------------
// Finalize
//-----------------------------------------------------------------------------
void CGame::Finalize( void )
{
	// GJEg
	m_Encount.Finalize();

	// TEh
	m_Sound.Finalize();

	// J[\폜
	m_Cursor.Delete();

	CUseItem::GetInstance()->Finalize();

	// EBhE폜
	m_wndMsg.Delete();
	m_wndYesNo.Delete();
	m_wndChoice.Delete();

	m_wndCamp.Delete();
	m_wndStore.Delete();


	// LN^G폜
	CDataManager::GetInstance()->DeleteCharacterGraph();
}
//-----------------------------------------------------------------------------
// ݂̃[h擾
//-----------------------------------------------------------------------------
int CGame::GetMode( void )
{
	return m_lstMode.back();
}
//-----------------------------------------------------------------------------
// [hX^bNNAĐݒ
//-----------------------------------------------------------------------------
void CGame::SetMode( int nMode )
{
	m_lstMode.clear();
	PushMode( nMode );
}
//-----------------------------------------------------------------------------
// [hX^bNɃvbV
//-----------------------------------------------------------------------------
void CGame::PushMode( int nMode )
{
	m_lstMode.push_back( nMode );
	LOG_OUTPUT("PushMode %d\n", nMode );
}
//-----------------------------------------------------------------------------
// [hX^bN|bv
//-----------------------------------------------------------------------------
int CGame::PopMode( void )
{
	int nMode = m_lstMode.back();
	m_lstMode.pop_back();

	LOG_OUTPUT("PopMode %d  %d\n", nMode, GetMode() );

	return nMode;
}
//-----------------------------------------------------------------------------
// UpdatePlayTime
//-----------------------------------------------------------------------------
void CGame::UpdatePlayTime( void )
{
	//               OvC   + ( ݎ     - X^[g )
	m_timPlayTime = m_timLastPlayTime + ( time( NULL ) - m_timStartTime );
}
//-----------------------------------------------------------------------------
// FrameMove
//-----------------------------------------------------------------------------
void CGame::FrameMove( void )
{
	UpdatePlayTime();

	// t[JE^҂
	if ( m_nWaitStatus == WAIT_STS_FRAME )
	{
		if ( 0 < m_nWaitFrame )
		{
			m_nWaitFrame --;
			m_Map.FrameMove( m_PlayerParty.GetScrollSpeed() );
			return;
		}
		SetWaitStatus( WAIT_STS_NONE );
		m_Map.ExecNext();
	}

	// GtFNg
	m_Effect.FrameMove();

	//  ҂
	// ANeBuł΃L[`FbN
	if ( GetActiveFlag() )
	{
		m_Mani.CheckKeyState();
	}


#if 0	// for debug
	if ( m_Mani.CheckKey( KEY_INPUT_A ) )
	{
		m_Effect.StartEncount();
	}
#endif

	switch ( GetMode() )
	{
	case MODE_MAP:	// n}ړ[h
#if 1	// for debug
	if ( m_Mani.CheckKey( KEY_INPUT_A ) )
	{
		Encount( ENCOUNT_ENEMY_RANDUM );
	}
#endif
		// bZ[W̍XV
		m_wndMsg.FrameMove( &m_Mani );
		m_wndYesNo.FrameMove( &m_Mani );
		m_wndChoice.FrameMove( &m_Mani );

		if ( !m_wndMsg.IsShow() || !m_wndYesNo.IsShow() || !m_wndChoice.IsShow() )
		{	// EBhE\Ȃn}͏Ȃ
			FrameMove_ModeMap();
		}

		//  [hؑփ`FbN
		if ( m_nWaitStatus == WAIT_STS_NONE && !m_Map.IsScrolling() )
		{
			// Xe[^Xʂ
			if ( m_Mani.CheckKey( KEY_INPUT_S ) )
			{
				m_wndCamp.SetMenuCursor( 0 );
				m_wndCamp.SetVisible( TRUE );
				PushMode( MODE_CAMP );
				return;
			}
		}
		break;

	case MODE_EVENT:	// Cxgs[h
		if ( m_wndMsg.IsShow() )
		{
			m_wndMsg.FrameMove( &m_Mani );
		}
		if ( m_wndYesNo.IsShow() )
		{
			m_wndYesNo.FrameMove( &m_Mani );
		}
		if ( m_wndChoice.IsShow() )
		{
			m_wndChoice.FrameMove( &m_Mani );
		}
		if ( m_wndStore.IsVisible() )
		{
			m_wndStore.FrameMove( &m_Mani );
		}
		break;
	case MODE_CAMP:
		// LvEBhE
		m_wndCamp.FrameMove( &m_Mani );
		break;

	//case MODE_STORE:
		//XEBhE
		//m_wndStore.FrameMove( &m_Mani );
		break;

	case MODE_BATTLE:
		// Cxg퓬I҂
		m_Battle.FrameMove( &m_Mani );
		break;
	default:
		break;
	}


	//  WaitStatus`FbN
	if (  m_nWaitStatus == WAIT_STS_BATTLE_START && !m_Effect.IsExecuting() )
	{
		SetWaitStatus( WAIT_STS_NONE );
		StartBattle();

	}
	// GtFNgI҂
	if ( m_nWaitStatus == WAIT_STS_SCR_EFFECT && !m_Effect.IsExecuting() )
	{
		SetWaitStatus( WAIT_STS_NONE );
		m_Map.ExecNext();
	}

	// bZ[WEBhE҂
	if ( m_nWaitStatus == WAIT_STS_SCR_MESSAGE_WND && m_wndMsg.IsWaitNext() )
	{
		SetWaitStatus( WAIT_STS_SCR_MESSAGE_WND );
		m_Map.ExecNext();
	}
	// I_CAO񓚑҂
	if ( m_nWaitStatus == WAIT_STS_YES_NO_ANSER && !m_wndYesNo.IsVisible() )
	{
		SetWaitStatus( WAIT_STS_NONE );
		m_Map.ExecNext();
	}
	// _CAO񓚑҂
	if ( m_nWaitStatus == WAIT_STS_CHOICE_ANSER && !m_wndChoice.IsVisible() )
	{
		SetWaitStatus( WAIT_STS_NONE );
		m_Map.ExecNext();
	}
	// }bv֑ؑ҂
	if ( m_nWaitStatus == WAIT_STS_MAP_CHANGE && !m_Map.IsScriptExecuting() )
	{
		SetWaitStatus( WAIT_STS_NONE );
		ChangeMap( &m_MapChangeReqParam );
	}

	// }bvXN[҂
	if ( m_nWaitStatus == WAIT_STS_MAP_SCROLL )
	{
		m_PlayerParty.FrameMove( &m_Map );
		if ( !m_Map.IsScrolling() )
		{
			m_PlayerParty.SetDirection( m_MapMoveReqParam.nDirection );
			if ( m_MapMoveReqParam.nDirection == DIRECTION_D )
			{
				m_Map.MoveOffsetTile( 0, 1, DIRECTION_D );
			}else if ( m_MapMoveReqParam.nDirection == DIRECTION_L )
			{
				m_Map.MoveOffsetTile( -1, 0, DIRECTION_L );
			}else if ( m_MapMoveReqParam.nDirection == DIRECTION_U )
			{
				m_Map.MoveOffsetTile( 0, -1, DIRECTION_U );
			}else if ( m_MapMoveReqParam.nDirection == DIRECTION_R )
			{
				m_Map.MoveOffsetTile( 1, 0, DIRECTION_R );
			}
			SetWaitStatus( WAIT_STS_NONE );
			m_Map.ExecNext();
		}
		m_Map.FrameMove( m_MapMoveReqParam.nSpeed );

	}

	// }bvIuWFNgANV҂
	if ( m_nWaitStatus == WAIT_STS_MAPOBJ_ACTION )
	{
		if ( m_pMapObjMoveReqTgt && !m_pMapObjMoveReqTgt->IsActing() )
		{
			//LOG_OUTPUT("!IsActing [%d]\n", m_pMapObjMoveReqTgt->IsActing() );
			SetWaitStatus( WAIT_STS_NONE );
			m_Map.ExecNext();
		}
		m_Map.FrameMove( m_PlayerParty.GetScrollSpeed() );
	}

	// XI҂
	if ( m_nWaitStatus == WAIT_STS_STORE_WND && !m_wndStore.IsVisible() )
	{
		SetWaitStatus( WAIT_STS_NONE );
		m_Map.ExecNext();
	}

	// J[\
	m_Cursor.FrameMove();
}
//-----------------------------------------------------------------------------
// FrameMove_ModeMap
//-----------------------------------------------------------------------------
void CGame::FrameMove_ModeMap( void )
{
	// _bV
	m_PlayerParty.SetDash( m_Mani.IsKeyPushed( KEY_INPUT_Z ) );

	//  L
	m_PlayerParty.FrameMove( &m_Mani, &m_Map );

	//  }bv
	if ( !m_Map.IsScrolling() )
	{
		// GJEg`FbN
		int nEnemyParty = m_Encount.CheckEncount();
		if ( nEnemyParty != CEncountManager::ENCOUNT_PARTY_NONE )
		{
			Encount( nEnemyParty );
			return;
		}


		// XN[̓L[Ȃ
		if ( m_Mani.IsKeyPushed( KEY_INPUT_UP ) )
		{
			m_Map.MoveOffsetTile( 0, -1, DIRECTION_U );
		}
		else if ( m_Mani.IsKeyPushed( KEY_INPUT_DOWN ) )
		{
			m_Map.MoveOffsetTile( 0, 1, DIRECTION_D );
		}
		else if ( m_Mani.IsKeyPushed( KEY_INPUT_LEFT ) )
		{
			m_Map.MoveOffsetTile( -1, 0, DIRECTION_L );
		}
		else if ( m_Mani.IsKeyPushed( KEY_INPUT_RIGHT ) )
		{
			m_Map.MoveOffsetTile( 1, 0, DIRECTION_R );
		}
	}
	m_Map.FrameMove( m_PlayerParty.GetScrollSpeed() );


}
//-----------------------------------------------------------------------------
// Render
// `
//-----------------------------------------------------------------------------
void CGame::Render( void )
{
	m_Effect.PreRender();

	// XN[ɕ`
	RenderToTargetScreen();

	/* C̃eXg
	for ( int iCnt = 0; iCnt < MAX_ANIMATION_CHIP; iCnt ++ )
	{
		for ( int iFrame = 0; iFrame < CHIP_ANIMATION_FRAME; iFrame ++ )
		{
			DrawGraph( iFrame * (MAP_CHIP_SIZE+2), 50+ iCnt * MAP_CHIP_SIZE, CDataManager::GetInstance()->m_nAniGraphHandle[iCnt][iFrame], FALSE );
		}
	}

	static UINT nFrameCnt = 0;
	int nAniNo = (nFrameCnt / 2) % (16*16);
	DrawGraph( 100, 100, CDataManager::GetInstance()->m_nAniGraphHandle[0][nAniNo], FALSE );

	nFrameCnt ++;
	*/

	m_Effect.PostRender();
}
//-----------------------------------------------------------------------------
// RenderToTargetScreen
// XN[`
//-----------------------------------------------------------------------------
void CGame::RenderToTargetScreen( void )
{
	if ( GetMode() == MODE_CAMP )
	{
		// LvEBhE
		m_wndCamp.Render();
		return;
	}
	if ( GetMode() == MODE_BATTLE )
	{
		// 퓬\
		m_Battle.Render();
		return;
	}

	if ( m_nWaitStatus == WAIT_STS_STORE_WND )
	{
		// XEBhE
		m_wndStore.Render();
		return;
	}

	using namespace std;

	// MAP 
	m_Map.RenderLayer(LAYER_0);

	// MAP 
	m_Map.RenderLayer(LAYER_1);

	// LN^`
	m_Map.RenderMapObjects( m_PlayerParty.GetDisplayPlayer() );

	// MAP 
	m_Map.RenderLayer(LAYER_2);

	// bZ[W`
	m_wndMsg.Render();
	m_wndYesNo.Render();
	m_wndChoice.Render();

	// J[\`
	m_Cursor.Render();

	// ʓGtFNg
	m_Effect.Render();

	// fobO
	int iX,iY;
	m_Map.GetCenterPos( &iX, &iY );
	DrawFormatString( 0 , 15 , GetColor( 255,255,255) , "[%d:%d]" , iX, iY );

}
//-----------------------------------------------------------------------------
// DEBUG_init
//-----------------------------------------------------------------------------
void CGame::DEBUG_init( void )
{
	SetMode( MODE_MAP );
	//SetMode( MODE_CAMP );
	m_wndCamp.SetVisible( TRUE );

	// }bvXNvgǍ
	int nInitMapNo = CSysDebug::GetDebugParam( "Init", "MapID", 2 );
	m_Map.Load( nInitMapNo );		// }bvǂݍ

	int nInitMapPosX = CSysDebug::GetDebugParam( "Init", "MapPosX", 7 );
	int nInitMapPosY = CSysDebug::GetDebugParam( "Init", "MapPosY", 7 );
	m_Map.SetCenterPos( nInitMapPosX, nInitMapPosY );

	// v[ǉ
	int nPlayerCount = CSysDebug::GetDebugParam( "Init", "PlayerCount", 4 );
	for ( int iCnt = 0; iCnt < nPlayerCount; iCnt ++ )
	{
		m_PlayerParty.AddPlayer( iCnt+1, iCnt+1 );
	}

	// z
	m_PlayerParty.AddItem( 2001 );	// iCt
	m_PlayerParty.AddItem( 3001 );	// ̂ӂ
	m_PlayerParty.AddItem( 2001 );	// iCt
	m_PlayerParty.AddItem( 3001 );	// ̂ӂ

	m_PlayerParty.AddItem( 1 );		// |[V
	m_PlayerParty.AddItem( 2002 );	// u[h\[h
	m_PlayerParty.AddItem( 2002 );	// u[h\[h
	m_PlayerParty.AddItem( 2003 );	// 
	m_PlayerParty.AddItem( 3001 );	// ̂ӂ
	m_PlayerParty.AddItem( 3002 );	// ̂ڂ
	
	int nInitGill = CSysDebug::GetDebugParam( "Init", "Gill", 2000 );
	m_PlayerParty.AddGill( nInitGill );	// Mǉ

	// ݒ
	// 1l
	CPlayerChar *pPlayer = m_PlayerParty.GetPlayer( 0 );
	stItemInfo *pInfo = m_PlayerParty.GetItemInfoFromItemID( 2001 );
	pPlayer->SetEquipItem( (CEquipItem *)pInfo->pItem, CHR_POS_DOMINANT );	// E
	pInfo = m_PlayerParty.GetItemInfoFromItemID( 3001 );
	pPlayer->SetEquipItem( (CEquipItem *)pInfo->pItem, CHR_POS_BODY );		// 
	//pPlayer->SetDirection( DIRECTION_U );
	pPlayer->SetDirection( DIRECTION_D );
	pPlayer->CalcParam();		// p^vZ

	// 2l
	pPlayer = m_PlayerParty.GetPlayer( 1 );
	pInfo = m_PlayerParty.GetItemInfoFromItemID( 2001 );
	pPlayer->SetEquipItem( (CEquipItem *)pInfo->pItem, CHR_POS_DOMINANT );	// E
	pInfo = m_PlayerParty.GetItemInfoFromItemID( 3001 );
	pPlayer->SetEquipItem( (CEquipItem *)pInfo->pItem, CHR_POS_BODY );		// 
	pPlayer->CalcParam();		// p^vZ

	pPlayer = m_PlayerParty.GetPlayer( 0 );
	pPlayer->m_CharInfo.nJobID = JOB_ID_SUPPIN;			// Wuݒ
	DEBUG_SetupHPMP( pPlayer );


	pPlayer = m_PlayerParty.GetPlayer( 1 );
	pPlayer->m_CharInfo.nJobID = JOB_ID_BLACK_WIZARD;	// Wuݒ
	DEBUG_SetupHPMP( pPlayer );

	pPlayer = m_PlayerParty.GetPlayer( 2 );
	pPlayer->m_CharInfo.nJobID = JOB_ID_KNIGHT;			// Wuݒ
	DEBUG_SetupHPMP( pPlayer );

	pPlayer = m_PlayerParty.GetPlayer( 3 );
	pPlayer->m_CharInfo.nJobID = JOB_ID_WHITE_WIZARD;	// Wuݒ
	DEBUG_SetupHPMP( pPlayer );

	// ACe\[g
	m_PlayerParty.SortItem();

}
//-----------------------------------------------------------------------------
// DEBUG_SetupHPMP
//-----------------------------------------------------------------------------
void CGame::DEBUG_SetupHPMP( CPlayerChar *pPlayer )
{
	char szKey[64];
	sprintf_s( szKey, "Player%d_Exp", pPlayer->GetID() );
	int nAddExp =CSysDebug::GetDebugParam( "Init", szKey, 0 );
	pPlayer->AddExp( nAddExp );

	int nPlayerHP = CSysDebug::GetDebugParam( "Init", "PlayerHP", 0 );
	int nPlayerMP = CSysDebug::GetDebugParam( "Init", "PlayerMP", 0 );

	pPlayer->RecoveryHPMP();	// HP/MPS
	// HP
	if ( nPlayerHP == 1 )
	{
		pPlayer->ChangeHitPoint( (int)(pPlayer->m_Param.Base.nMaxHP / 2) );
	}
	else if ( nPlayerHP == 2 )
	{
		pPlayer->m_CharInfo.nHP = 0;
	}
	// MP
	if ( nPlayerMP == 1 )
	{
		pPlayer->ChangeMagicPoint( (int)(pPlayer->m_Param.Base.nMaxMP / 2) );
	}
	else if ( nPlayerMP == 2 )
	{
		pPlayer->m_CharInfo.nMP = 0;
	}
}

//-----------------------------------------------------------------------------
// ChangeDirToPlayer
//-----------------------------------------------------------------------------
void CGame::ChangeDirToPlayer( CMapObject *pMapObj )
{
	int nDirection;

	// v[Ɣ΂
	switch ( m_PlayerParty.GetDisplayPlayer()->GetDirection() )
	{
	case DIRECTION_D:
		nDirection = DIRECTION_U;
		break;
	case DIRECTION_U:
		nDirection = DIRECTION_D;
		break;
	case DIRECTION_L:
		nDirection = DIRECTION_R;
		break;
	case DIRECTION_R:
		nDirection = DIRECTION_L;
		break;
	default:
		nDirection = DIRECTION_D;
		break;
	}

	pMapObj->SetDirection( nDirection );
}
//-----------------------------------------------------------------------------
// StartEventBattle
//-----------------------------------------------------------------------------
void CGame::StartEventBattle( int nEnemyPartyID )
{
	m_bEventBattle = TRUE;
	Encount( nEnemyPartyID );
}
//-----------------------------------------------------------------------------
// Encount
//-----------------------------------------------------------------------------
void CGame::Encount( int nEnemyPartyID )
{
	m_Effect.StartEncount();

	m_nEnemyPartyID = nEnemyPartyID;
	SetWaitStatus( WAIT_STS_BATTLE_START );
}
//-----------------------------------------------------------------------------
// StartBattle
// 퓬p[^̐ݒ
//-----------------------------------------------------------------------------
void CGame::StartBattle( void )
{
	m_Battle.Delete();

	BATTLE_PARAM Param;

	// for debug
	Param.nFiledID = 1;
	Param.pPlayerParty = &m_PlayerParty;

	// Gݒ
	if ( m_nEnemyPartyID == ENCOUNT_ENEMY_RANDUM )
	{	// _
		Param.nEnemyPartyID = GetRand(1000) % 18 + 1;		// ToDo ForDebug
	}
	else
	{
		Param.nEnemyPartyID = m_nEnemyPartyID;
	}

	m_Battle.Create( &Param );
	m_Battle.SetVisible( TRUE );

	PushMode( MODE_BATTLE );
}

//-----------------------------------------------------------------------------
// CanSaveNow
// Z[u\
//-----------------------------------------------------------------------------
BOOL CGame::CanSaveNow( void )
{
	// [h}bv\Z[u|Cg̏̂݉\
	if ( GetMode() != MODE_MAP )
		return FALSE;

	if ( m_Map.GetID() == MAP_ID_WORLD )
		return TRUE;

	if ( m_Map.IsPlayerOnSavePoint() )
		return TRUE;

	return FALSE;
}
//-----------------------------------------------------------------------------
// FinishedBattle
// 퓬̖߂菈
//-----------------------------------------------------------------------------
void CGame::FinishedBattle( void )
{
	// BGMĐ~
	m_Sound.StopAllBGM();

	// BGMĐ
	int nBGMID = m_Map.GetBGMID();
	m_Sound.PlayBGM( nBGMID, FALSE );

	PopMode();
	if ( GetMode() == MODE_EVENT )
	{
		m_Map.ExecNext();
	}
}

//-----------------------------------------------------------------------------
// StartScriptRequest
// XNvgJnv
//-----------------------------------------------------------------------------
void CGame::StartScriptRequest( CMapObject *pMapObj )
{
	m_Map.StartExec( pMapObj );
}

//-----------------------------------------------------------------------------
// ShowMessage
// bZ[WEBhE\
//-----------------------------------------------------------------------------
void CGame::ShowMessage( const char *lpzMessage )
{
	// v[̕
	CMapObject *pObj = m_Map.GetTargetMapObject();
	if ( pObj && pObj->GetType() != MAP_OBJECT_TYPE_STABLE )
	{	// ToDo:
		ChangeDirToPlayer( pObj );
	}

	// 
	m_wndMsg.SetMessage( lpzMessage );
	m_wndMsg.ShowWindow( TRUE );

	// EBhE҂
	SetWaitStatus( WAIT_STS_SCR_MESSAGE_WND );
}
//-----------------------------------------------------------------------------
// CloseMessage
// bZ[WEBhE
//-----------------------------------------------------------------------------
void CGame::CloseMessage( void )
{
	m_wndMsg.ShowWindow( FALSE );

	// XNvgsÕ[hɖ߂
	//SetToPrevMode();
	SetWaitStatus( WAIT_STS_NONE );
}
//-----------------------------------------------------------------------------
// AddMessage
// bZ[W1sǉ
//-----------------------------------------------------------------------------
void CGame::AddMessage( const char *lpzMessage )
{
	// 擾
	m_wndMsg.AddMessage( lpzMessage );
	m_wndMsg.ShowWindow( TRUE );

	// EBhE҂
	SetWaitStatus( WAIT_STS_SCR_MESSAGE_WND );
}
//-----------------------------------------------------------------------------
// StartFadeout
// tF[hAEg
//-----------------------------------------------------------------------------
void CGame::StartFadeout( DWORD dwColor, UINT nFrame )
{
	m_Effect.StartFadeout( dwColor, nFrame );
	SetWaitStatus( WAIT_STS_SCR_EFFECT );
}
//-----------------------------------------------------------------------------
// StartFadein
// tF[hC
//-----------------------------------------------------------------------------
void CGame::StartFadein( DWORD dwColor, UINT nFrame )
{
	m_Effect.StartFadein( dwColor, nFrame );
	SetWaitStatus( WAIT_STS_SCR_EFFECT );

}
//-----------------------------------------------------------------------------
// StartShake
// VFCN
//-----------------------------------------------------------------------------
void CGame::StartShake( int nType, UINT nFrame, double dblSpeed, int nScale )
{
	m_Effect.StartShake( nType, nFrame, dblSpeed, nScale );
	SetWaitStatus( WAIT_STS_SCR_EFFECT );
}

//-----------------------------------------------------------------------------
// ChangeMapRequest
//-----------------------------------------------------------------------------
void CGame::ChangeMapRequest( int nMapID, int nLocX, int nLocY, int nPlayerDir )
{
	m_MapChangeReqParam.nMapID	   = nMapID;
	m_MapChangeReqParam.nLocX	   = nLocX;
	m_MapChangeReqParam.nLocY	   = nLocY;
	m_MapChangeReqParam.nPlayerDir = nPlayerDir;

	SetWaitStatus( WAIT_STS_MAP_CHANGE );
}
//-----------------------------------------------------------------------------
// ChangeMapRequest
//-----------------------------------------------------------------------------
void CGame::ChangeMap( stMapChangeReqParam *pPram )
{
	m_PlayerParty.GetDisplayPlayer()->SetPosTile( SCREEN_CENTER_TILE_X, SCREEN_CENTER_TILE_Y, pPram->nPlayerDir );

	m_Map.Load( pPram->nMapID );		// }bvǂݍ
	m_Map.SetCenterPos( pPram->nLocX, pPram->nLocY );
}

//-----------------------------------------------------------------------------
// GetTreasureFlag
//-----------------------------------------------------------------------------
char CGame::GetTreasureFlag( int nMapID, int nEventID, int *pIndex /*= NULL */ )
{
	stTreasureData *pTreasure = CFlag::GetTreasureData( nMapID, nEventID );
	if ( pTreasure->nIndex < 0 )
		return -1;
	if ( pIndex )
	{
		*pIndex = pTreasure->nIndex;
	}

	return m_Flag.m_chTreasure[pTreasure->nIndex];
}
//-----------------------------------------------------------------------------
// GetTreasure
//-----------------------------------------------------------------------------
void CGame::GetTreasure( int nMapID, int nEventID, CItem **pItem  )
{
	stTreasureData *pTreasure = CFlag::GetTreasureData( nMapID, nEventID );
	if ( pTreasure->nIndex < 0 )
		return;

	// tO擾ς݂
	m_Flag.m_chTreasure[pTreasure->nIndex] = 1;

	// ACef[^擾
	CDataManager::GetInstance()->LoadOneItemData( pTreasure->nIndex, pItem );

	// }bvIuWFNg̊G؂ւ
	CMapObject *pObj = m_Map.GetTargetMapObject();
	pObj->SetState( CCharacterObject::STATE_2 );

	// p[eB̃ACeɒǉ
	m_PlayerParty.AddItem( (*pItem)->nID );
}
//-----------------------------------------------------------------------------
// MovePlayer
//-----------------------------------------------------------------------------
void CGame::MovePlayer( int nDirection, int nSpeed )
{
	m_MapMoveReqParam.nDirection = nDirection;
	m_MapMoveReqParam.nSpeed = nSpeed;

	// }bvXN[҂
	SetWaitStatus( WAIT_STS_MAP_SCROLL );
}
//-----------------------------------------------------------------------------
// MoveMapObject
//-----------------------------------------------------------------------------
void CGame::MoveMapObject( int nEventID, int nDirection, int nSpeed )
{
	CMapObject *pMapObj = m_pMapObjMoveReqTgt = m_Map.GetMapObject( nEventID );
	if ( !pMapObj )
		return;

	int nFrameMove;
	switch ( nSpeed )
	{
	case SCROLL_SPEED_WALK:
		nFrameMove = 32;
		break;
	case SCROLL_SPEED_DASH:
		nFrameMove = 16;
		break;
	default:
		nFrameMove = 32;
	}

	pMapObj->StartMove( nDirection, nFrameMove );

	// }bvIuWFNgANV҂
	SetWaitStatus( WAIT_STS_MAPOBJ_ACTION );
}

//-----------------------------------------------------------------------------
// WaitFrame
//-----------------------------------------------------------------------------
void CGame::WaitFrame( int nFrame )
{
	m_nWaitFrame = nFrame;
	SetWaitStatus( WAIT_STS_FRAME );
}
//-----------------------------------------------------------------------------
// OpenStore
//-----------------------------------------------------------------------------
void CGame::OpenStore( int nType, int nMsgPtn, int nRate, int nItemCount, int *pItemID )
{
	//SetMode( MODE_STORE );
	m_wndStore.SetStoreType( nType, nMsgPtn );
	m_wndStore.SetStoreItem( nItemCount, pItemID, nRate );
	m_wndStore.SetVisible( TRUE );

	SetWaitStatus( WAIT_STS_STORE_WND );
}
//-----------------------------------------------------------------------------
// LoadSoundFile
//-----------------------------------------------------------------------------
void CGame::LoadSoundFile( int nID )
{
	m_Sound.LoadMemory( nID );

}
//-----------------------------------------------------------------------------
// PlayBGM
//-----------------------------------------------------------------------------
void CGame::PlayBGM( int nID, BOOL bTopPositionFlag )
{
	m_Sound.PlayBGM( nID, bTopPositionFlag );
}
//-----------------------------------------------------------------------------
// StopBGM
//-----------------------------------------------------------------------------
void CGame::StopBGM( void )
{
	m_Sound.StopAllBGM();
}
//-----------------------------------------------------------------------------
// GetGameFlag
//-----------------------------------------------------------------------------
char CGame::GetGameFlag( int nIndex )
{
	if ( nIndex < 0 || GAME_FLAG_MAX <= nIndex )
		return 0;

	return m_Flag.m_chGame[nIndex];
}
//-----------------------------------------------------------------------------
// SetGameFlag
//-----------------------------------------------------------------------------
void CGame::SetGameFlag( int nIndex, char chValue )
{
	if ( nIndex < 0 || GAME_FLAG_MAX <= nIndex )
		return;

	m_Flag.m_chGame[nIndex] = chValue;
}
//-----------------------------------------------------------------------------
// ShowYesNoMessage
//-----------------------------------------------------------------------------
void CGame::ShowYesNoMessage( const char *lpzMessage )
{
	m_wndYesNo.SetQuestionMessage( lpzMessage );
	m_wndYesNo.SetVisible( TRUE );

	SetWaitStatus( WAIT_STS_YES_NO_ANSER );
}
//-----------------------------------------------------------------------------
// ShowChoiceMessage
//-----------------------------------------------------------------------------
void CGame::ShowChoiceMessage( const char *lpzQuestion, const char *lpzAns1, const char *lpzAns2, const char *lpzAns3 )
{
	m_wndChoice.SetQuestionMessage( lpzQuestion, lpzAns1, lpzAns2, lpzAns3 );
	m_wndChoice.SetVisible( TRUE );

	SetWaitStatus( WAIT_STS_CHOICE_ANSER );
}
//-----------------------------------------------------------------------------
// GetYesNoAnswer
//-----------------------------------------------------------------------------
int CGame::GetYesNoAnswer( void )
{
	return m_wndYesNo.GetChooseAns();
}
//-----------------------------------------------------------------------------
// GetChooseAnswer
//-----------------------------------------------------------------------------
int CGame::GetChooseAnswer( void )
{
	return m_wndChoice.GetChooseAns();
}