//=============================================================================
// MenuWindow.cpp
//=============================================================================
#include "Stdinc.h"
#include "MenuWindow.h"
#include "Game.h"
#include "Cursor.h"
#include "Manipulator.h"

//-----------------------------------------------------------------------------
// RXgN^
//-----------------------------------------------------------------------------
CMenuWindow::CMenuWindow(void)
	: m_nCursorIndex ( 0 ), m_bFocused( FALSE ), m_bLoopAtEdge( FALSE )
{
	m_pCursor = CGame::GetInstance()->GetCursor();
}

//-----------------------------------------------------------------------------
// fXgN^
//-----------------------------------------------------------------------------
CMenuWindow::~CMenuWindow(void)
{
	ClearMenuItem();
}
//-----------------------------------------------------------------------------
// ACeǉ stMenuItem\̌^
//-----------------------------------------------------------------------------
void CMenuWindow::AddMenuItem( const stMenuItem *pItem )
{
	AddMenuItem( pItem->nID, pItem->bEnabled, pItem->nPosX, pItem->nPosY, pItem->lpzText, pItem->nIcon, pItem->nCount );
}
//-----------------------------------------------------------------------------
// ACeǉ
//-----------------------------------------------------------------------------
void CMenuWindow::AddMenuItem( int nID, BOOL bEnabled, int nPosX, int nPosY, const char *lpzText, int nIcon, int nCount )
{
	stMenuItem *pNewMenuItem = new stMenuItem();
	pNewMenuItem->nID = nID;
	pNewMenuItem->bEnabled = bEnabled;
	pNewMenuItem->nPosX = nPosX;
	pNewMenuItem->nPosY = nPosY;

	int nLen = strlen( lpzText );
	pNewMenuItem->lpzText = new char [nLen+1];
	strcpy_s( pNewMenuItem->lpzText, nLen+1, lpzText );

	pNewMenuItem->nIcon = nIcon;
	pNewMenuItem->nCount = nCount;

	// Xgɒǉ
	m_aryItem.push_back( pNewMenuItem );
}
//-----------------------------------------------------------------------------
// ACeXV
//-----------------------------------------------------------------------------
void CMenuWindow::UpdateMenuText( int nID, BOOL bEnabled, const char *lpzText, int nIcon, int nCount )
{
	stMenuItem *pNewMenuItem = m_aryItem.at( nID );
	pNewMenuItem->bEnabled = bEnabled;

	// XV
	delete [] pNewMenuItem->lpzText;

	int nLen = strlen( lpzText );
	pNewMenuItem->lpzText = new char [nLen+1];
	strcpy_s( pNewMenuItem->lpzText, nLen+1, lpzText );

	pNewMenuItem->nIcon = nIcon;
	pNewMenuItem->nCount = nCount;

}
//-----------------------------------------------------------------------------
// ACeFXV
//-----------------------------------------------------------------------------
void CMenuWindow::EnableMenuItem( int nID, BOOL bEnabled )
{
	stMenuItem *pNewMenuItem = m_aryItem.at( nID );
	pNewMenuItem->bEnabled = bEnabled;
}
//-----------------------------------------------------------------------------
// j[NA
//-----------------------------------------------------------------------------
void CMenuWindow::ClearMenuItem( void )
{
	using namespace std;
	
	for ( int iCnt = 0; iCnt < (int)m_aryItem.size(); iCnt ++ )
	{
		stMenuItem *pItem = m_aryItem.at( iCnt );

		delete [] pItem->lpzText;
		delete pItem;
	}
	m_aryItem.clear();
}
//-----------------------------------------------------------------------------
// ACe擾
//-----------------------------------------------------------------------------
stMenuItem *CMenuWindow::GetItem( int nIndex )
{
	return m_aryItem.at( nIndex );
}
//-----------------------------------------------------------------------------
// J[\ʒuݒ
//-----------------------------------------------------------------------------
int CMenuWindow::SetCurPos( int nIndex )
{
	// ͈̓`FbN
	if ( m_aryItem.size() == 0 || (int)m_aryItem.size() <= nIndex )
		return m_nCursorIndex;

	int nLastCursor = m_nCursorIndex;
	m_nCursorIndex = nIndex;

	stMenuItem *pItem = m_aryItem.at(nIndex);

	// J[\ւ͐΍WōW
	m_pCursor->SetPos( m_nPosX + pItem->nPosX - CURSOR_POS_X_MARGINE , m_nPosY + pItem->nPosY );

	return m_nCursorIndex;
}
//-----------------------------------------------------------------------------
// FrameMove
//-----------------------------------------------------------------------------
void CMenuWindow::FrameMove( const CManipulator *pMani )
{
	CFrameWindow::FrameMove();

	// tH[JX̎
	if ( !m_bFocused )
		return;

	// ΉL[ݒ
	int nKeyInc, nKeyDec;
	if ( m_nType == MENU_TYPE_VERTICAL )
	{
		nKeyInc = KEY_INPUT_DOWN;	// L[
		nKeyDec = KEY_INPUT_UP;		// L[
	}
	else
	{
		nKeyInc = KEY_INPUT_RIGHT;	// L[
		nKeyDec = KEY_INPUT_LEFT;	// L[
	}

	// L[ɂJ[\̈ړ
	if ( pMani->CheckKey( nKeyInc ) )
	{
		if ( m_nCursorIndex < (int)m_aryItem.size() - 1 )
		{	// 1
			SetCurPos( m_nCursorIndex + 1 );
		}
		else if ( m_bLoopAtEdge )
		{	// 1ԏ
			SetCurPos( 0 );
		}
	}
	else if ( pMani->CheckKey( nKeyDec ) )
	{
		if ( 0 < m_nCursorIndex )
		{	// 1
			SetCurPos( m_nCursorIndex - 1 );
		}
		else if ( m_bLoopAtEdge )
		{	// 1ԉ
			SetCurPos( (int)m_aryItem.size() - 1 );
		}
	}
}

//-----------------------------------------------------------------------------
// Render
// `
//-----------------------------------------------------------------------------
void CMenuWindow::Render( void )
{
	if ( !IsVisible() )
		return;

	CFrameWindow::Render();

	using namespace std;
	for ( int iCnt = 0; iCnt < (int)m_aryItem.size(); iCnt ++ )
	{
		DWORD dwColor;
		stMenuItem *pItem = m_aryItem.at( iCnt );
		if ( pItem->bEnabled )
		{
			dwColor = STRING_COLOR;
		}
		else
		{
			dwColor = DISABLED_STRING_COLOR;
		}
		DrawString( m_nPosX + pItem->nPosX, m_nPosY + pItem->nPosY, pItem->lpzText, dwColor );
	}

	// tH[JXȂJ[\`
	if ( m_bFocused )
	{
		m_pCursor->Render();
	}
}
